@@ -41,6 +41,7 @@ cross_bootstrap=0
 auth_root_authentication_method=socket
 auth_root_socket_user=""
 skip_test_db=0
+extra_file=""
 
 dirname0=`dirname $0 2>/dev/null`
 dirname0=`dirname $dirname0 2>/dev/null`
@@ -101,6 +102,8 @@ Usage: $0 [OPTIONS]
                        group. You must be root to use this option.  By default
                        mysqld runs using your current group and files and
                        directories that it creates will be owned by you.
+  --extra-file=file    Add user defined SQL file, to be executed following
+                       regular database initialization.
 
 All other options are passed to the mysqld program
 
@@ -186,6 +189,8 @@ parse_arguments()
       --auth-root-socket-user=*)
         auth_root_socket_user="$(parse_arg "$arg")" ;;
       --skip-test-db) skip_test_db=1 ;;
+      --extra-file=*)
+        extra_file="$(parse_arg "$arg")" ;;
 
       *)
         if test -n "$pick_args"
@@ -407,6 +412,13 @@ do
   fi
 done
 
+# Verify extra file exists if it's not null
+if test ! -z "$extra_file" -a ! -f "$extra_file"
+then
+  cannot_find_file "$extra_file"
+  exit 1
+fi
+
 if test ! -x "$mysqld"
 then
   cannot_find_file "$mysqld"
@@ -493,7 +505,7 @@ done
 
 if test -n "$user"
 then
-  if test -z "$srcdir" -a "$in_rpm" -eq 0 -a -d "$pamtooldir/auth_pam_tool_dir"
+  if test -z "$srcdir" -a "$in_rpm" -eq 0
   then
     chown 0 "$pamtooldir/auth_pam_tool_dir/auth_pam_tool" && \
     chmod 04755 "$pamtooldir/auth_pam_tool_dir/auth_pam_tool"
@@ -538,25 +550,6 @@ else
   filter_cmd_line="cat"
 fi
 
-# Disable log error if the user don't have right to write/create the file
-# This is common when a user tries to install a personal mariadbd server and
-# the global config in /etc is using --log-error.
-# The server will internally change log-error to stderr to stderr if it cannot
-# write the the log file. This code only disables the error message from a not
-# writable log-error, which can be confusing.
-if test -n "$log_error"
-then
-    if test \( -e "$log_error" -a \! -w "$log_error" \) -o \( ! -e "$log_error" -a ! -w "`dirname "$log_error"`" \)
-    then
-        if test -n "$verbose"
-        then
-            echo "resetting log-error '$log_error' because no write access"
-        fi
-        log_error=""
-        args="$args --skip-log-error"
-    fi
-fi
-
 # Configure mysqld command line
 mysqld_bootstrap="${MYSQLD_BOOTSTRAP-$mysqld}"
 mysqld_install_cmd_line()
@@ -592,6 +585,12 @@ cat_sql()
   then
     cat "$mysql_test_db"
   fi
+
+  # cat extra file if it's not null
+  if test ! -z "$extra_file"
+  then
+    cat "$extra_file"
+  fi
 }
 
 # Create the system and help tables by passing them to "mysqld --bootstrap"
@@ -629,7 +628,7 @@ else
   echo "--general-log gives you a log in $ldata that may be helpful."
   link_to_help
   echo "You can find the latest source at https://downloads.mariadb.org and"
-  echo "the MariaDB discuss email list at https://lists.mariadb.org/postorius/lists/discuss.lists.mariadb.org/"
+  echo "the maria-discuss email list at https://launchpad.net/~maria-discuss"
   echo
   echo "Please check all of the above before submitting a bug report"
   echo "at https://mariadb.org/jira"
@@ -677,7 +676,7 @@ then
   then
     echo
     echo "You can start the MariaDB daemon with:"
-    echo "cd '$basedir' ; $bindir/mariadbd-safe --datadir='$ldata'"
+    echo "cd '$basedir' ; $bindir/mariadb-safe --datadir='$ldata'"
     echo
     echo "You can test the MariaDB daemon with mysql-test-run.pl"
     echo "cd '$basedir/@INSTALL_MYSQLTESTDIR@' ; perl mariadb-test-run.pl"
