@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
 
 /**
   @file Representation of an SQL command.
@@ -38,7 +38,7 @@ enum enum_sql_command {
   SQLCOM_SHOW_DATABASES, SQLCOM_SHOW_TABLES, SQLCOM_SHOW_FIELDS,
   SQLCOM_SHOW_KEYS, SQLCOM_SHOW_VARIABLES, SQLCOM_SHOW_STATUS,
   SQLCOM_SHOW_ENGINE_LOGS, SQLCOM_SHOW_ENGINE_STATUS, SQLCOM_SHOW_ENGINE_MUTEX,
-  SQLCOM_SHOW_PROCESSLIST, SQLCOM_SHOW_MASTER_STAT, SQLCOM_SHOW_SLAVE_STAT,
+  SQLCOM_SHOW_PROCESSLIST, SQLCOM_SHOW_BINLOG_STAT, SQLCOM_SHOW_SLAVE_STAT,
   SQLCOM_SHOW_GRANTS, SQLCOM_SHOW_CREATE, SQLCOM_SHOW_CHARSETS,
   SQLCOM_SHOW_COLLATIONS, SQLCOM_SHOW_CREATE_DB, SQLCOM_SHOW_TABLE_STATUS,
   SQLCOM_SHOW_TRIGGERS,
@@ -75,7 +75,6 @@ enum enum_sql_command {
   SQLCOM_XA_START, SQLCOM_XA_END, SQLCOM_XA_PREPARE,
   SQLCOM_XA_COMMIT, SQLCOM_XA_ROLLBACK, SQLCOM_XA_RECOVER,
   SQLCOM_SHOW_PROC_CODE, SQLCOM_SHOW_FUNC_CODE,
-  SQLCOM_ALTER_TABLESPACE,
   SQLCOM_INSTALL_PLUGIN, SQLCOM_UNINSTALL_PLUGIN,
   SQLCOM_SHOW_AUTHORS, SQLCOM_BINLOG_BASE64_EVENT,
   SQLCOM_SHOW_PLUGINS, SQLCOM_SHOW_CONTRIBUTORS,
@@ -108,6 +107,7 @@ enum enum_sql_command {
   SQLCOM_SHOW_STATUS_PACKAGE,
   SQLCOM_SHOW_STATUS_PACKAGE_BODY,
   SQLCOM_SHOW_PACKAGE_BODY_CODE,
+  SQLCOM_BACKUP, SQLCOM_BACKUP_LOCK,
 
   /*
     When a command is added here, be sure it's also added in mysqld.cc
@@ -117,6 +117,32 @@ enum enum_sql_command {
   SQLCOM_END
 };
 
+
+class Storage_engine_name
+{
+protected:
+  LEX_CSTRING m_storage_engine_name;
+public:
+  Storage_engine_name()
+  {
+    m_storage_engine_name.str= NULL;
+    m_storage_engine_name.length= 0;
+  }
+  Storage_engine_name(const LEX_CSTRING &name)
+   :m_storage_engine_name(name)
+  { }
+  Storage_engine_name(const LEX_STRING &name)
+  {
+    m_storage_engine_name.str= name.str;
+    m_storage_engine_name.length= name.length;
+  }
+  bool resolve_storage_engine_with_error(THD *thd,
+                                         handlerton **ha,
+                                         bool tmp_table);
+  bool is_set() { return m_storage_engine_name.str != NULL; }
+};
+
+
 /**
   @class Sql_cmd - Representation of an SQL command.
 
@@ -160,9 +186,13 @@ public:
   */
   virtual bool execute(THD *thd) = 0;
 
+  virtual Storage_engine_name *option_storage_engine_name()
+  {
+    return NULL;
+  }
+
 protected:
-  Sql_cmd()
-  {}
+  Sql_cmd() = default;
 
   virtual ~Sql_cmd()
   {
@@ -176,6 +206,46 @@ protected:
   }
 };
 
+class Sql_cmd_show_slave_status: public Sql_cmd
+{
+protected:
+  bool show_all_slaves_status;
+public:
+  Sql_cmd_show_slave_status()
+    :show_all_slaves_status(false)
+  {}
+
+  Sql_cmd_show_slave_status(bool status_all)
+    :show_all_slaves_status(status_all)
+  {}
+
+  enum_sql_command sql_command_code() const { return SQLCOM_SHOW_SLAVE_STAT; }
+
+  bool execute(THD *thd);
+  bool is_show_all_slaves_stat() { return show_all_slaves_status; }
+};
+
+
+class Sql_cmd_create_table_like: public Sql_cmd,
+                                 public Storage_engine_name
+{
+public:
+  Storage_engine_name *option_storage_engine_name() { return this; }
+  bool execute(THD *thd);
+};
+
+class Sql_cmd_create_table: public Sql_cmd_create_table_like
+{
+public:
+  enum_sql_command sql_command_code() const { return SQLCOM_CREATE_TABLE; }
+};
+
+class Sql_cmd_create_sequence: public Sql_cmd_create_table_like
+{
+public:
+  enum_sql_command sql_command_code() const { return SQLCOM_CREATE_SEQUENCE; }
+};
+
 
 /**
   Sql_cmd_call represents the CALL statement.
@@ -190,8 +260,7 @@ public:
     m_handler(handler)
   {}
 
-  virtual ~Sql_cmd_call()
-  {}
+  virtual ~Sql_cmd_call() = default;
 
   /**
     Execute a CALL statement at runtime.
@@ -206,5 +275,4 @@ public:
   }
 };
 
-
 #endif // SQL_CMD_INCLUDED
