@@ -56,7 +56,7 @@
     invoked on a running DELETE statement.
 */
 
-Explain_delete* Delete_plan::save_explain_delete_data(THD *thd, MEM_ROOT *mem_root)
+Explain_delete* Delete_plan::save_explain_delete_data(MEM_ROOT *mem_root, THD *thd)
 {
   Explain_query *query= thd->lex->explain;
   Explain_delete *explain= 
@@ -73,7 +73,7 @@ Explain_delete* Delete_plan::save_explain_delete_data(THD *thd, MEM_ROOT *mem_ro
   else
   {
     explain->deleting_all_rows= false;
-    if (Update_plan::save_explain_data_intern(thd, mem_root, explain,
+    if (Update_plan::save_explain_data_intern(mem_root, explain,
                                               thd->lex->analyze_stmt))
       return 0;
   }
@@ -84,22 +84,21 @@ Explain_delete* Delete_plan::save_explain_delete_data(THD *thd, MEM_ROOT *mem_ro
 
 
 Explain_update* 
-Update_plan::save_explain_update_data(THD *thd, MEM_ROOT *mem_root)
+Update_plan::save_explain_update_data(MEM_ROOT *mem_root, THD *thd)
 {
   Explain_query *query= thd->lex->explain;
   Explain_update* explain= 
     new (mem_root) Explain_update(mem_root, thd->lex->analyze_stmt);
   if (!explain)
     return 0;
-  if (save_explain_data_intern(thd, mem_root, explain, thd->lex->analyze_stmt))
+  if (save_explain_data_intern(mem_root, explain, thd->lex->analyze_stmt))
     return 0;
   query->add_upd_del_plan(explain);
   return explain;
 }
 
 
-bool Update_plan::save_explain_data_intern(THD *thd,
-                                           MEM_ROOT *mem_root,
+bool Update_plan::save_explain_data_intern(MEM_ROOT *mem_root,
                                            Explain_update *explain,
                                            bool is_analyze)
 {
@@ -121,16 +120,9 @@ bool Update_plan::save_explain_data_intern(THD *thd,
     return 0;
   }
   
-  if (is_analyze ||
-      (thd->variables.log_slow_verbosity &
-       LOG_SLOW_VERBOSITY_ENGINE))
-  {
+  if (is_analyze)
     table->file->set_time_tracker(&explain->table_tracker);
 
-    if (table->file->handler_stats && table->s->tmp_table != INTERNAL_TMP_TABLE)
-      explain->handler_for_stats= table->file;
-  }
-
   select_lex->set_explain_type(TRUE);
   explain->select_type= select_lex->type;
   /* Partitions */
@@ -263,10 +255,7 @@ int update_portion_of_time(THD *thd, TABLE *table,
     res= src->save_in_field(table->field[dst_fieldno], true);
 
   if (likely(!res))
-  {
-    table->period_prepare_autoinc();
     res= table->update_generated_fields();
-  }
 
   if(likely(!res))
     res= table->file->ha_update_row(table->record[1], table->record[0]);
@@ -398,10 +387,9 @@ bool mysql_delete(THD *thd, TABLE_LIST *table_list, COND *conds,
   if (returning)
     (void) result->prepare(returning->item_list, NULL);
 
-  if (!thd->lex->current_select->leaf_tables_saved)
+  if (thd->lex->current_select->first_cond_optimization)
   {
     thd->lex->current_select->save_leaf_tables(thd);
-    thd->lex->current_select->leaf_tables_saved= true;
     thd->lex->current_select->first_cond_optimization= 0;
   }
   /* check ORDER BY even if it can be ignored */
@@ -438,10 +426,10 @@ bool mysql_delete(THD *thd, TABLE_LIST *table_list, COND *conds,
     DBUG_RETURN(TRUE);
   }
 
-  const_cond_result= const_cond && (!conds || conds->val_bool());
+  const_cond_result= const_cond && (!conds || conds->val_int());
   if (unlikely(thd->is_error()))
   {
-    /* Error evaluating val_bool(). */
+    /* Error evaluating val_int(). */
     DBUG_RETURN(TRUE);
   }
 
@@ -485,7 +473,7 @@ bool mysql_delete(THD *thd, TABLE_LIST *table_list, COND *conds,
       query_type= THD::STMT_QUERY_TYPE;
       error= -1;
       deleted= maybe_deleted;
-      if (!query_plan.save_explain_delete_data(thd, thd->mem_root))
+      if (!query_plan.save_explain_delete_data(thd->mem_root, thd))
         error= 1;
       goto cleanup;
     }
@@ -520,13 +508,6 @@ bool mysql_delete(THD *thd, TABLE_LIST *table_list, COND *conds,
     if (thd->lex->describe || thd->lex->analyze_stmt)
       goto produce_explain_and_leave;
 
-    if (!thd->lex->current_select->leaf_tables_saved)
-    {
-      thd->lex->current_select->save_leaf_tables(thd);
-      thd->lex->current_select->leaf_tables_saved= true;
-      thd->lex->current_select->first_cond_optimization= 0;
-    }
-
     my_ok(thd, 0);
     DBUG_RETURN(0);
   }
@@ -541,8 +522,7 @@ bool mysql_delete(THD *thd, TABLE_LIST *table_list, COND *conds,
   select=make_select(table, 0, 0, conds, (SORT_INFO*) 0, 0, &error);
   if (unlikely(error))
     DBUG_RETURN(TRUE);
-  if ((select && select->check_quick(thd, safe_update, limit,
-                                     Item_func::BITMAP_ALL)) || !limit)
+  if ((select && select->check_quick(thd, safe_update, limit)) || !limit)
   {
     query_plan.set_impossible_where();
     if (thd->lex->describe || thd->lex->analyze_stmt)
@@ -558,14 +538,6 @@ bool mysql_delete(THD *thd, TABLE_LIST *table_list, COND *conds,
     */
     if (unlikely(thd->is_error()))
       DBUG_RETURN(TRUE);
-
-    if (!thd->lex->current_select->leaf_tables_saved)
-    {
-      thd->lex->current_select->save_leaf_tables(thd);
-      thd->lex->current_select->leaf_tables_saved= true;
-      thd->lex->current_select->first_cond_optimization= 0;
-    }
-
     my_ok(thd, 0);
     DBUG_RETURN(0);				// Nothing to delete
   }
@@ -622,7 +594,7 @@ bool mysql_delete(THD *thd, TABLE_LIST *table_list, COND *conds,
   if (thd->lex->describe)
     goto produce_explain_and_leave;
   
-  if (!(explain= query_plan.save_explain_delete_data(thd, thd->mem_root)))
+  if (!(explain= query_plan.save_explain_delete_data(thd->mem_root, thd)))
     goto got_error;
   ANALYZE_START_TRACKING(thd, &explain->command_tracker);
 
@@ -742,8 +714,8 @@ bool mysql_delete(THD *thd, TABLE_LIST *table_list, COND *conds,
     table->mark_columns_needed_for_delete();
   }
 
-  if (!table->prepare_triggers_for_delete_stmt_or_event() &&
-      table->file->ha_table_flags() & HA_CAN_FORCE_BULK_DELETE)
+  if ((table->file->ha_table_flags() & HA_CAN_FORCE_BULK_DELETE) &&
+      !table->prepare_triggers_for_delete_stmt_or_event())
     will_batch= !table->file->start_bulk_delete();
 
   /*
@@ -761,6 +733,8 @@ bool mysql_delete(THD *thd, TABLE_LIST *table_list, COND *conds,
   explain= (Explain_delete*)thd->lex->explain->get_upd_del_plan();
   explain->tracker.on_scan_init();
 
+  thd->get_stmt_da()->reset_current_row_for_warning(1);
+
   if (!delete_while_scanning)
   {
     /*
@@ -773,21 +747,27 @@ bool mysql_delete(THD *thd, TABLE_LIST *table_list, COND *conds,
                                              MEM_STRIP_BUF_SIZE);
 
     THD_STAGE_INFO(thd, stage_searching_rows_for_update);
-    while (!(error=info.read_record()) && !thd->killed && !thd->is_error())
+    while (!(error=info.read_record()) && !thd->killed &&
+          ! thd->is_error())
     {
       if (record_should_be_deleted(thd, table, select, explain, delete_history))
       {
         table->file->position(table->record[0]);
-        if ((error= deltempfile->unique_add((char*) table->file->ref)))
-          break;
+        if (unlikely((error=
+                      deltempfile->unique_add((char*) table->file->ref))))
+        {
+          error= 1;
+          goto terminate_delete;
+        }
         if (!--tmplimit && using_limit)
           break;
       }
     }
     end_read_record(&info);
-    if (table->file->ha_index_or_rnd_end() || error > 0 ||
-        deltempfile->get(table) ||
-        init_read_record(&info, thd, table, 0, &deltempfile->sort, 0, 1, 0))
+    if (unlikely(deltempfile->get(table)) ||
+        unlikely(table->file->ha_index_or_rnd_end()) ||
+        unlikely(init_read_record(&info, thd, table, 0, &deltempfile->sort, 0,
+                                  1, false)))
     {
       error= 1;
       goto terminate_delete;
@@ -820,9 +800,11 @@ bool mysql_delete(THD *thd, TABLE_LIST *table_list, COND *conds,
   THD_STAGE_INFO(thd, stage_updating);
   fix_rownum_pointers(thd, thd->lex->current_select, &deleted);
 
+  thd->get_stmt_da()->reset_current_row_for_warning(0);
   while (likely(!(error=info.read_record())) && likely(!thd->killed) &&
          likely(!thd->is_error()))
   {
+    thd->get_stmt_da()->inc_current_row_for_warning();
     if (delete_while_scanning)
       delete_record= record_should_be_deleted(thd, table, select, explain,
                                               delete_history);
@@ -898,6 +880,7 @@ bool mysql_delete(THD *thd, TABLE_LIST *table_list, COND *conds,
     else
       break;
   }
+  thd->get_stmt_da()->reset_current_row_for_warning(1);
 
 terminate_delete:
   killed_status= thd->killed;
@@ -927,10 +910,9 @@ cleanup:
     query_cache_invalidate3(thd, table_list, 1);
   }
 
-  if (!thd->lex->current_select->leaf_tables_saved)
+  if (thd->lex->current_select->first_cond_optimization)
   {
     thd->lex->current_select->save_leaf_tables(thd);
-    thd->lex->current_select->leaf_tables_saved= true;
     thd->lex->current_select->first_cond_optimization= 0;
   }
 
@@ -1000,7 +982,7 @@ produce_explain_and_leave:
     We come here for various "degenerate" query plans: impossible WHERE,
     no-partitions-used, impossible-range, etc.
   */
-  if (!(query_plan.save_explain_delete_data(thd, thd->mem_root)))
+  if (!(query_plan.save_explain_delete_data(thd->mem_root, thd)))
     goto got_error;
 
 send_nothing_and_leave:
@@ -1117,11 +1099,10 @@ int mysql_prepare_delete(THD *thd, TABLE_LIST *table_list, Item **conds,
 ***************************************************************************/
 
 
-extern "C" int refpos_order_cmp(void *arg, const void *a, const void *b)
+extern "C" int refpos_order_cmp(void* arg, const void *a,const void *b)
 {
-  auto file= static_cast<handler *>(arg);
-  return file->cmp_ref(static_cast<const uchar *>(a),
-                       static_cast<const uchar *>(b));
+  handler *file= (handler*)arg;
+  return file->cmp_ref((const uchar*)a, (const uchar*)b);
 }
 
 /*
@@ -1271,13 +1252,6 @@ multi_delete::initialize_tables(JOIN *join)
   {
     TABLE_LIST *tbl= walk->correspondent_table->find_table_for_update();
     tables_to_delete_from|= tbl->table->map;
-
-    /*
-      Ensure that filesort re-reads the row from the engine before
-      delete is called.
-    */
-    join->map2table[tbl->table->tablenr]->keep_current_rowid= true;
-
     if (delete_while_scanning &&
         unique_table(thd, tbl, join->tables_list, 0))
     {
@@ -1438,13 +1412,6 @@ void multi_delete::abort_result_set()
 {
   DBUG_ENTER("multi_delete::abort_result_set");
 
-  /****************************************************************************
-
-    NOTE: if you change here be aware that almost the same code is in
-     multi_delete::send_eof().
-
-  ***************************************************************************/
-
   /* the error was handled or nothing deleted and no side effects return */
   if (error_handled ||
       (!thd->transaction->stmt.modified_non_trans_table && !deleted))
@@ -1645,13 +1612,6 @@ bool multi_delete::send_eof()
   /* reset used flags */
   THD_STAGE_INFO(thd, stage_end);
 
-  /****************************************************************************
-
-    NOTE: if you change here be aware that almost the same code is in
-     multi_delete::abort_result_set().
-
-  ***************************************************************************/
-
   if (thd->transaction->stmt.modified_non_trans_table)
     thd->transaction->all.modified_non_trans_table= TRUE;
   thd->transaction->all.m_unsafe_rollback_flags|=
