@@ -24,7 +24,8 @@
 #include "my_json_writer.h"
 #include "opt_range.h"
 #include "sql_expression_cache.h"
-#include "item_subselect.h"
+
+#include <stack>
 
 const char * STR_DELETING_ALL_ROWS= "Deleting all rows";
 const char * STR_IMPOSSIBLE_WHERE= "Impossible WHERE";
@@ -42,7 +43,7 @@ static void write_item(Json_writer *writer, Item *item);
 static void append_item_to_str(String *out, Item *item);
 
 Explain_query::Explain_query(THD *thd_arg, MEM_ROOT *root) : 
-  mem_root(root), upd_del_plan(NULL),  insert_plan(NULL),
+  mem_root(root), upd_del_plan(nullptr),  insert_plan(nullptr),
   unions(root), selects(root),  thd(thd_arg), apc_enabled(false),
   operations(0)
 {
@@ -208,9 +209,6 @@ int Explain_query::send_explain(THD *thd, bool extended)
 int Explain_query::print_explain(select_result_sink *output, 
                                  uint8 explain_flags, bool is_analyze)
 {
-  /* A sanity check for ANALYZE: */
-  DBUG_ASSERT(timer_tracker_frequency() != 0.0);
-
   if (upd_del_plan)
   {
     upd_del_plan->print_explain(this, output, explain_flags, is_analyze);
@@ -786,18 +784,6 @@ bool Explain_node::print_explain_json_cache(Json_writer *writer,
 }
 
 
-bool Explain_node::print_explain_json_subq_materialization(Json_writer *writer,
-                                                           bool is_analyze)
-{
-  if (subq_materialization)
-  {
-    subq_materialization->print_explain_json(writer, is_analyze);
-    return true;
-  }
-  return false;
-}
-
-
 Explain_basic_join::~Explain_basic_join()
 {
   if (join_tabs)
@@ -945,8 +931,6 @@ void Explain_select::print_explain_json(Explain_query *query,
   Json_writer_nesting_guard guard(writer);
   
   bool started_cache= print_explain_json_cache(writer, is_analyze);
-  bool started_subq_mat= print_explain_json_subq_materialization(writer,
-                                                                 is_analyze);
 
   if (message ||
       select_type == pushed_derived_text ||
@@ -1050,8 +1034,6 @@ void Explain_select::print_explain_json(Explain_query *query,
     writer->end_object();
   }
 
-  if (started_subq_mat)
-    writer->end_object();
   if (started_cache)
     writer->end_object();
 }
@@ -1110,14 +1092,13 @@ void Explain_aggr_window_funcs::print_json_members(Json_writer *writer,
 {
   Explain_aggr_filesort *srt;
   List_iterator<Explain_aggr_filesort> it(sorts);
-  writer->add_member("sorts").start_object();
+  Json_writer_array sorts(writer, "sorts");
   while ((srt= it++))
   {
-    writer->add_member("filesort").start_object();
+    Json_writer_object sort(writer);
+    Json_writer_object filesort(writer, "filesort");
     srt->print_json_members(writer, is_analyze);
-    writer->end_object(); // filesort
   }
-  writer->end_object(); // sorts
 }
 
 
@@ -1139,17 +1120,26 @@ print_explain_json_interns(Explain_query *query,
                            Json_writer *writer, 
                            bool is_analyze)
 {
-  Json_writer_nesting_guard guard(writer);
-  for (uint i=0; i< n_join_tabs; i++)
   {
-    if (join_tabs[i]->start_dups_weedout)
-      writer->add_member("duplicates_removal").start_object();
+    Json_writer_array loop(writer, "nested_loop");
+    for (uint i=0; i< n_join_tabs; i++)
+    {
+      if (join_tabs[i]->start_dups_weedout)
+      {
+        writer->start_object();
+        writer->add_member("duplicates_removal");
+        writer->start_array();
+      }
 
-    join_tabs[i]->print_explain_json(query, writer, is_analyze);
+      join_tabs[i]->print_explain_json(query, writer, is_analyze);
 
-    if (join_tabs[i]->end_dups_weedout)
-      writer->end_object();
-  }
+      if (join_tabs[i]->end_dups_weedout)
+      {
+        writer->end_array();
+        writer->end_object();
+      }
+    }
+  } // "nested_loop"
   print_explain_json_for_children(query, writer, is_analyze);
 }
 
@@ -1729,34 +1719,12 @@ void Explain_rowid_filter::print_explain_json(Explain_query *query,
   writer->end_object(); // rowid_filter
 }
 
-static void trace_engine_stats(handler *file, Json_writer *writer)
-{
-  if (file && file->handler_stats)
-  {
-    ha_handler_stats *hs= file->handler_stats;
-    writer->add_member("r_engine_stats").start_object();
-    if (hs->pages_accessed)
-      writer->add_member("pages_accessed").add_ull(hs->pages_accessed);
-    if (hs->pages_updated)
-      writer->add_member("pages_updated").add_ull(hs->pages_updated);
-    if (hs->pages_read_count)
-      writer->add_member("pages_read_count").add_ull(hs->pages_read_count);
-    if (hs->pages_read_time)
-      writer->add_member("pages_read_time_ms").
-        add_double(hs->pages_read_time * 1000. / timer_tracker_frequency());
-    if (hs->pages_prefetched)
-      writer->add_member("pages_prefetch_read_count").add_ull(hs->pages_prefetched);
-    if (hs->undo_records_read)
-      writer->add_member("old_rows_read").add_ull(hs->undo_records_read);
-    writer->end_object();
-  }
-}
 
 void Explain_table_access::print_explain_json(Explain_query *query,
                                               Json_writer *writer,
                                               bool is_analyze)
 {
-  Json_writer_nesting_guard guard(writer);
+  Json_writer_object jsobj(writer);
   
   if (pre_join_sort)
   {
@@ -1892,7 +1860,6 @@ void Explain_table_access::print_explain_json(Explain_query *query,
       writer->add_member("r_table_time_ms").add_double(total_time);
       writer->add_member("r_other_time_ms").add_double(extra_time_tracker.get_time_ms());
     }
-    trace_engine_stats(handler_for_stats, writer);
   }
 
   /* `filtered` */
@@ -1959,11 +1926,6 @@ void Explain_table_access::print_explain_json(Explain_query *query,
 
       writer->add_member("r_unpack_time_ms");
       writer->add_double(jbuf_unpack_tracker.get_time_ms());
-      DBUG_EXECUTE_IF("analyze_print_r_unpack_ops",
-                      {
-                        writer->add_member("r_unpack_ops");
-                        writer->add_ull(jbuf_unpack_tracker.get_loops());
-                      });
 
       writer->add_member("r_other_time_ms").
         add_double(jbuf_extra_time_tracker.get_time_ms());
@@ -2181,14 +2143,15 @@ void Explain_quick_select::print_json(Json_writer *writer)
   }
   else
   {
-    writer->add_member(get_name_by_type()).start_object();
+    Json_writer_array ranges(writer, get_name_by_type());
 
     List_iterator_fast<Explain_quick_select> it (children);
     Explain_quick_select* child;
     while ((child = it++))
+    {
+      Json_writer_object obj(writer);
       child->print_json(writer);
-
-    writer->end_object();
+    }
   }
 }
 
@@ -2599,8 +2562,6 @@ void Explain_update::print_explain_json(Explain_query *query,
     }
   }
 
-  trace_engine_stats(handler_for_stats, writer);
-
   if (where_cond)
   {
     writer->add_member("attached_condition");
@@ -2775,41 +2736,3 @@ void Explain_range_checked_fer::print_json(Json_writer *writer,
     writer->end_object();
   }
 }
-
-
-void Explain_subq_materialization::print_explain_json(Json_writer *writer,
-                                                      bool is_analyze)
-{
-  writer->add_member("materialization").start_object();
-  if (is_analyze)
-    tracker.print_json_members(writer);
-}
-
-
-void Subq_materialization_tracker::print_json_members(Json_writer *writer) const
-{
-  writer->add_member("r_strategy").add_str(get_exec_strategy());
-  if (loops_count)
-    writer->add_member("r_loops").add_ull(loops_count);
-
-  if (index_lookups_count)
-    writer->add_member("r_index_lookups").add_ull(index_lookups_count);
-
-  if (partial_matches_count)
-    writer->add_member("r_partial_matches").add_ull(partial_matches_count);
-
-  if (partial_match_buffer_size)
-  {
-    writer->add_member("r_partial_match_buffer_size").
-            add_size(partial_match_buffer_size);
-  }
-
-  if (partial_match_array_sizes.elements())
-  {
-    writer->add_member("r_partial_match_array_sizes").start_array();
-    for(size_t i= 0; i < partial_match_array_sizes.elements(); i++)
-      writer->add_ull(partial_match_array_sizes[i]);
-    writer->end_array();
-  }
-}
-
