@@ -190,7 +190,7 @@
 /* The rest of the file is included in the server only */
 #ifndef MYSQL_CLIENT
 
-/* @@optimizer_switch flags. These must be in sync with optimizer_switch_names */
+/* @@optimizer_switch flags. These must be in sync with optimizer_switch_typelib */
 #define OPTIMIZER_SWITCH_INDEX_MERGE               (1ULL << 0)
 #define OPTIMIZER_SWITCH_INDEX_MERGE_UNION         (1ULL << 1)
 #define OPTIMIZER_SWITCH_INDEX_MERGE_SORT_UNION    (1ULL << 2)
@@ -234,6 +234,7 @@
 #define OPTIMIZER_SWITCH_USE_ROWID_FILTER          (1ULL << 33)
 #define OPTIMIZER_SWITCH_COND_PUSHDOWN_FROM_HAVING (1ULL << 34)
 #define OPTIMIZER_SWITCH_NOT_NULL_RANGE_SCAN       (1ULL << 35)
+#define OPTIMIZER_SWITCH_HASH_JOIN_CARDINALITY     (1ULL << 36)
 
 #define OPTIMIZER_SWITCH_DEFAULT   (OPTIMIZER_SWITCH_INDEX_MERGE | \
                                     OPTIMIZER_SWITCH_INDEX_MERGE_UNION | \
@@ -266,11 +267,6 @@
                                     OPTIMIZER_SWITCH_COND_PUSHDOWN_FROM_HAVING | \
                                     OPTIMIZER_SWITCH_OPTIMIZE_JOIN_BUFFER_SIZE)
 
-/*
-  See adjust_secondary_key_cost in sys_vars.cc for symbolic names.
-*/
-#define OPTIMIZER_ADJ_FIX_CARD_MULT (1)
-
 /*
   Replication uses 8 bytes to store SQL_MODE in the binary log. The day you
   use strictly more than 64 bits by adding one more define above, you should
@@ -321,10 +317,7 @@
 */
 /* This subquery has fields from outer query (put by user) */
 #define UNCACHEABLE_DEPENDENT_GENERATED   1
-/*
-  This subquery contains functions with random result.
-  Something that is uncacheable is by default unmergeable.
-*/
+/* This subquery contains functions with random result */
 #define UNCACHEABLE_RAND        2
 /* This subquery contains functions with side effect */
 #define UNCACHEABLE_SIDEEFFECT	4
@@ -342,6 +335,8 @@
 #define UNCACHEABLE_DEPENDENT (UNCACHEABLE_DEPENDENT_GENERATED | \
                                UNCACHEABLE_DEPENDENT_INJECTED)
 
+#define FAKE_SELECT_LEX_ID UINT_MAX
+
 /* Used to check GROUP BY list in the MODE_ONLY_FULL_GROUP_BY mode */
 #define UNDEF_POS (-1)
 
@@ -425,16 +420,6 @@ inline int hexchar_to_int(char c)
 /* This must match the path length limit in the ER_NOT_RW_DIR error msg. */
 #define ER_NOT_RW_DIR_PATHSIZE 200
 
-#define IS_TABLESPACES_TABLESPACE_NAME    0
-#define IS_TABLESPACES_ENGINE             1
-#define IS_TABLESPACES_TABLESPACE_TYPE    2
-#define IS_TABLESPACES_LOGFILE_GROUP_NAME 3
-#define IS_TABLESPACES_EXTENT_SIZE        4
-#define IS_TABLESPACES_AUTOEXTEND_SIZE    5
-#define IS_TABLESPACES_MAXIMUM_SIZE       6
-#define IS_TABLESPACES_NODEGROUP_ID       7
-#define IS_TABLESPACES_TABLESPACE_COMMENT 8
-
 bool db_name_is_in_ignore_db_dirs_list(const char *dbase);
 
 #endif /* MYSQL_SERVER */
