@@ -60,6 +60,10 @@ static constexpr ulint BUF_LRU_OLD_TOLERANCE = 20;
 frames in the buffer pool, we set this to TRUE */
 static bool buf_lru_switched_on_innodb_mon = false;
 
+/** True if diagnostic message about difficult to find free blocks
+in the buffer bool has already printed. */
+static bool	buf_lru_free_blocks_error_printed;
+
 /******************************************************************//**
 These statistics are not 'of' LRU but 'for' LRU.  We keep count of I/O
 and page_zip_decompress() operations.  Based on the statistics,
@@ -385,94 +389,123 @@ we put it to free list to be used.
 @return the free control block, in state BUF_BLOCK_MEMORY */
 buf_block_t *buf_LRU_get_free_block(bool have_mutex)
 {
-  bool waited= false;
-  MONITOR_INC(MONITOR_LRU_GET_FREE_SEARCH);
-  if (!have_mutex)
-    mysql_mutex_lock(&buf_pool.mutex);
+	ulint		n_iterations	= 0;
+	ulint		flush_failures	= 0;
+	MONITOR_INC(MONITOR_LRU_GET_FREE_SEARCH);
+	if (have_mutex) {
+		mysql_mutex_assert_owner(&buf_pool.mutex);
+		goto got_mutex;
+	}
+	DBUG_EXECUTE_IF("recv_ran_out_of_buffer",
+			if (recv_recovery_is_on()
+			    && recv_sys.apply_log_recs) {
+				mysql_mutex_lock(&buf_pool.mutex);
+				goto flush_lru;
+			});
+get_mutex:
+	mysql_mutex_lock(&buf_pool.mutex);
+got_mutex:
+	buf_LRU_check_size_of_non_data_objects();
+	buf_block_t* block;
 
-  buf_LRU_check_size_of_non_data_objects();
-
-  buf_block_t *block;
+	DBUG_EXECUTE_IF("ib_lru_force_no_free_page",
+		if (!buf_lru_free_blocks_error_printed) {
+			n_iterations = 21;
+			goto not_found;});
 
 retry:
-  /* If there is a block in the free list, take it */
-  block= buf_LRU_get_free_only();
-  if (block)
-  {
+	/* If there is a block in the free list, take it */
+	if ((block = buf_LRU_get_free_only()) != nullptr) {
 got_block:
-    const ulint LRU_size= UT_LIST_GET_LEN(buf_pool.LRU);
-    const ulint available= UT_LIST_GET_LEN(buf_pool.free);
-    const ulint scan_depth= srv_LRU_scan_depth / 2;
-    ut_ad(LRU_size <= BUF_LRU_MIN_LEN ||
-          available >= scan_depth || buf_pool.need_LRU_eviction());
+		if (!have_mutex) {
+			mysql_mutex_unlock(&buf_pool.mutex);
+		}
+		block->page.zip.clear();
+		return block;
+	}
 
-    ut_d(bool signalled = false);
+	MONITOR_INC( MONITOR_LRU_GET_FREE_LOOPS );
+	if (n_iterations || buf_pool.try_LRU_scan) {
+		/* If no block was in the free list, search from the
+		end of the LRU list and try to free a block there.
+		If we are doing for the first time we'll scan only
+		tail of the LRU list otherwise we scan the whole LRU
+		list. */
+		if (buf_LRU_scan_and_free_block(n_iterations
+						? ULINT_UNDEFINED : 100)) {
+			goto retry;
+		}
 
-    if (UNIV_UNLIKELY(available < scan_depth) && LRU_size > BUF_LRU_MIN_LEN)
-    {
-      mysql_mutex_lock(&buf_pool.flush_list_mutex);
-      if (!buf_pool.page_cleaner_active())
-      {
-        buf_pool.page_cleaner_wakeup(true);
-        ut_d(signalled = true);
-      }
-      mysql_mutex_unlock(&buf_pool.flush_list_mutex);
-    }
+		/* Tell other threads that there is no point
+		in scanning the LRU list. */
+		buf_pool.try_LRU_scan = false;
+	}
 
-    if (!have_mutex)
-      mysql_mutex_unlock(&buf_pool.mutex);
+	for (;;) {
+		if ((block = buf_LRU_get_free_only()) != nullptr) {
+			goto got_block;
+		}
+		mysql_mutex_unlock(&buf_pool.mutex);
+		mysql_mutex_lock(&buf_pool.flush_list_mutex);
+		const auto n_flush = buf_pool.n_flush();
+		if (!buf_pool.try_LRU_scan) {
+			buf_pool.page_cleaner_wakeup(true);
+		}
+		mysql_mutex_unlock(&buf_pool.flush_list_mutex);
+		mysql_mutex_lock(&buf_pool.mutex);
+		if (!n_flush) {
+			goto not_found;
+		}
+		if (!buf_pool.try_LRU_scan) {
+			my_cond_wait(&buf_pool.done_free,
+				     &buf_pool.mutex.m_mutex);
+		}
+	}
 
-    DBUG_EXECUTE_IF("ib_free_page_sleep",
-    {
-      static bool do_sleep = true;
-      if (do_sleep && signalled)
-      {
-        std::this_thread::sleep_for(std::chrono::milliseconds(10));
-        do_sleep = false;
-      }
-    });
+not_found:
+	if (n_iterations > 1) {
+		MONITOR_INC( MONITOR_LRU_GET_FREE_WAITS );
+	}
 
-    block->page.zip.clear();
-    return block;
-  }
+	if (n_iterations == 21 && !buf_lru_free_blocks_error_printed
+	    && srv_buf_pool_old_size == srv_buf_pool_size) {
+		buf_lru_free_blocks_error_printed = true;
+		mysql_mutex_unlock(&buf_pool.mutex);
+		ib::warn() << "Difficult to find free blocks in the buffer pool"
+			" (" << n_iterations << " search iterations)! "
+			<< flush_failures << " failed attempts to"
+			" flush a page!"
+			" Consider increasing innodb_buffer_pool_size."
+			" Pending flushes (fsync): "
+			<< fil_n_pending_tablespace_flushes
+			<< ". " << os_n_file_reads << " OS file reads, "
+			<< os_n_file_writes << " OS file writes, "
+			<< os_n_fsyncs
+			<< " OS fsyncs.";
+		mysql_mutex_lock(&buf_pool.mutex);
+	}
 
-  MONITOR_INC(MONITOR_LRU_GET_FREE_LOOPS);
-  if (waited || buf_pool.try_LRU_scan)
-  {
-    /* If no block was in the free list, search from the end of the
-    LRU list and try to free a block there.  If we are doing for the
-    first time we'll scan only tail of the LRU list otherwise we scan
-    the whole LRU list. */
-    if (buf_LRU_scan_and_free_block(waited ? ULINT_UNDEFINED : 100))
-      goto retry;
+	/* No free block was found: try to flush the LRU list.
+	The freed blocks will be up for grabs for all threads.
 
-    /* Tell other threads that there is no point in scanning the LRU
-    list. */
-    buf_pool.try_LRU_scan= false;
-  }
+	TODO: A more elegant way would have been to return one freed
+	up block to the caller here but the code that deals with
+	removing the block from buf_pool.page_hash and buf_pool.LRU is fairly
+	involved (particularly in case of ROW_FORMAT=COMPRESSED pages). We
+	can do that in a separate patch sometime in future. */
+#ifndef DBUG_OFF
+flush_lru:
+#endif
+	if (!buf_flush_LRU(innodb_lru_flush_size, true)) {
+		MONITOR_INC(MONITOR_LRU_SINGLE_FLUSH_FAILURE_COUNT);
+		++flush_failures;
+	}
 
-  waited= true;
-
-  while (!(block= buf_LRU_get_free_only()))
-  {
-    buf_pool.stat.LRU_waits++;
-
-    timespec abstime;
-    set_timespec(abstime, 1);
-
-    mysql_mutex_lock(&buf_pool.flush_list_mutex);
-    if (!buf_pool.page_cleaner_active())
-      buf_pool.page_cleaner_wakeup(true);
-    mysql_mutex_unlock(&buf_pool.flush_list_mutex);
-    if (my_cond_timedwait(&buf_pool.done_free, &buf_pool.mutex.m_mutex,
-                          &abstime))
-    {
-      buf_pool.LRU_warn();
-      buf_LRU_check_size_of_non_data_objects();
-    }
-  }
-
-  goto got_block;
+	n_iterations++;
+	buf_pool.stat.LRU_waits++;
+	mysql_mutex_unlock(&buf_pool.mutex);
+	buf_dblwr.flush_buffered_writes();
+	goto get_mutex;
 }
 
 /** Move the LRU_old pointer so that the length of the old blocks list
@@ -754,14 +787,6 @@ void buf_page_make_young(buf_page_t *bpage)
   mysql_mutex_unlock(&buf_pool.mutex);
 }
 
-bool buf_page_make_young_if_needed(buf_page_t *bpage)
-{
-  const bool not_first{bpage->set_accessed()};
-  if (UNIV_UNLIKELY(buf_page_peek_if_too_old(bpage)))
-    buf_page_make_young(bpage);
-  return not_first;
-}
-
 /** Try to free a block. If bpage is a descriptor of a compressed-only
 ROW_FORMAT=COMPRESSED page, the buf_page_t object will be freed as well.
 The caller must hold buf_pool.mutex.
@@ -824,13 +849,7 @@ relocate_compressed:
 		mysql_mutex_lock(&buf_pool.flush_list_mutex);
 		new (b) buf_page_t(*bpage);
 		b->frame = nullptr;
-		{
-			ut_d(uint32_t s=) b->fix();
-			ut_ad(s == buf_page_t::FREED
-			      || s == buf_page_t::UNFIXED
-			      || s == buf_page_t::IBUF_EXIST
-			      || s == buf_page_t::REINIT);
-		}
+		b->set_state(buf_page_t::UNFIXED + 1);
 		break;
 	default:
 		if (zip || !bpage->zip.data || !bpage->frame) {
@@ -1065,57 +1084,55 @@ static bool buf_LRU_block_remove_hashed(buf_page_t *bpage, const page_id_t id,
 
 	buf_pool.freed_page_clock += 1;
 
-	if (UNIV_LIKELY(!bpage->zip.data)) {
-		MEM_CHECK_ADDRESSABLE(bpage, sizeof(buf_block_t));
-		MEM_CHECK_ADDRESSABLE(bpage->frame, srv_page_size);
-		buf_block_modify_clock_inc((buf_block_t*) bpage);
-	} else if (const page_t *page = bpage->frame) {
+	if (UNIV_LIKELY(bpage->frame != nullptr)) {
 		MEM_CHECK_ADDRESSABLE(bpage, sizeof(buf_block_t));
 		MEM_CHECK_ADDRESSABLE(bpage->frame, srv_page_size);
 		buf_block_modify_clock_inc((buf_block_t*) bpage);
+		if (UNIV_LIKELY_NULL(bpage->zip.data)) {
+			const page_t*	page = bpage->frame;
 
-		ut_a(!zip || !bpage->oldest_modification());
-		ut_ad(bpage->zip_size());
-		/* Skip consistency checks if the page was freed.
-		In recovery, we could get a sole FREE_PAGE record
-		and nothing else, for a ROW_FORMAT=COMPRESSED page.
-		Its contents would be garbage. */
-		if (!bpage->is_freed())
-		switch (fil_page_get_type(page)) {
-		case FIL_PAGE_TYPE_ALLOCATED:
-		case FIL_PAGE_INODE:
-		case FIL_PAGE_IBUF_BITMAP:
-		case FIL_PAGE_TYPE_FSP_HDR:
-		case FIL_PAGE_TYPE_XDES:
-			/* These are essentially uncompressed pages. */
-			if (!zip) {
-				/* InnoDB writes the data to the
-				uncompressed page frame.  Copy it
-				to the compressed page, which will
-				be preserved. */
-				memcpy(bpage->zip.data, page,
-				       bpage->zip_size());
+			ut_a(!zip || !bpage->oldest_modification());
+			ut_ad(bpage->zip_size());
+
+			switch (fil_page_get_type(page)) {
+			case FIL_PAGE_TYPE_ALLOCATED:
+			case FIL_PAGE_INODE:
+			case FIL_PAGE_IBUF_BITMAP:
+			case FIL_PAGE_TYPE_FSP_HDR:
+			case FIL_PAGE_TYPE_XDES:
+				/* These are essentially uncompressed pages. */
+				if (!zip) {
+					/* InnoDB writes the data to the
+					uncompressed page frame.  Copy it
+					to the compressed page, which will
+					be preserved. */
+					memcpy(bpage->zip.data, page,
+					       bpage->zip_size());
+				}
+				break;
+			case FIL_PAGE_TYPE_ZBLOB:
+			case FIL_PAGE_TYPE_ZBLOB2:
+			case FIL_PAGE_INDEX:
+			case FIL_PAGE_RTREE:
+				break;
+			default:
+				ib::error() << "The compressed page to be"
+					" evicted seems corrupt:";
+				ut_print_buf(stderr, page, srv_page_size);
+
+				ib::error() << "Possibly older version of"
+					" the page:";
+
+				ut_print_buf(stderr, bpage->zip.data,
+					     bpage->zip_size());
+				putc('\n', stderr);
+				ut_error;
 			}
-			break;
-		case FIL_PAGE_TYPE_ZBLOB:
-		case FIL_PAGE_TYPE_ZBLOB2:
-		case FIL_PAGE_INDEX:
-		case FIL_PAGE_RTREE:
-			break;
-		default:
-			ib::error() << "The compressed page to be"
-				" evicted seems corrupt:";
-			ut_print_buf(stderr, page, srv_page_size);
-
-			ib::error() << "Possibly older version of"
-				" the page:";
-
-			ut_print_buf(stderr, bpage->zip.data,
-				     bpage->zip_size());
-			putc('\n', stderr);
-			ut_error;
+		} else {
+			goto evict_zip;
 		}
 	} else {
+evict_zip:
 		ut_a(!bpage->oldest_modification());
 		MEM_CHECK_ADDRESSABLE(bpage->zip.data, bpage->zip_size());
 	}
@@ -1155,6 +1172,25 @@ static bool buf_LRU_block_remove_hashed(buf_page_t *bpage, const page_id_t id,
 			return true;
 		}
 
+		/* Question: If we release hash_lock here
+		then what protects us against:
+		1) Some other thread buffer fixing this page
+		2) Some other thread trying to read this page and
+		not finding it in buffer pool attempting to read it
+		from the disk.
+		Answer:
+		1) Cannot happen because the page is no longer in the
+		page_hash. Only possibility is when while invalidating
+		a tablespace we buffer fix the prev_page in LRU to
+		avoid relocation during the scan. But that is not
+		possible because we are holding buf_pool mutex.
+
+		2) Not possible because in buf_page_init_for_read()
+		we do a look up of page_hash while holding buf_pool
+		mutex and since we are holding buf_pool mutex here
+		and by the time we'll release it in the caller we'd
+		have inserted the compressed only descriptor in the
+		page_hash. */
 		hash_lock.unlock();
 
 		if (bpage->zip.data) {
@@ -1188,7 +1224,6 @@ void buf_pool_t::corrupted_evict(buf_page_t *bpage, uint32_t state)
   buf_pool_t::hash_chain &chain= buf_pool.page_hash.cell_get(id.fold());
   page_hash_latch &hash_lock= buf_pool.page_hash.lock_get(chain);
 
-  recv_sys.free_corrupted_page(id);
   mysql_mutex_lock(&mutex);
   hash_lock.lock();
 
@@ -1213,6 +1248,8 @@ void buf_pool_t::corrupted_evict(buf_page_t *bpage, uint32_t state)
     buf_LRU_block_free_hashed_page(reinterpret_cast<buf_block_t*>(bpage));
 
   mysql_mutex_unlock(&mutex);
+
+  recv_sys.free_corrupted_page(id);
 }
 
 /** Update buf_pool.LRU_old_ratio.
